##############################################################################
#
# Copyright (c) 2008-2009 SIA "KN dati". (http://kndati.lv) All Rights Reserved.
#                    General contacts <info@kndati.lv>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import fields,osv
import tools

class acquisition(osv.osv):
	_name = "it.acquisition"
	_rec_name = "acq_doc_name"

	_columns = {
		'acq_doc_name': fields.char('Document name', size=32, required=True),
		'acq_document': fields.binary('Acquisition document'),
		'acq_date': fields.date('Acquisition date', required=True),
		'supplier': fields.many2one('res.partner', 'Supplier', required=True),
	}

acquisition()

class url(osv.osv):
	_name = "it.url"
	_rec_name = "name"

	_columns = {
		'name': fields.char('URL', size=256, required=True),
		'description': fields.char('Description', size=500, required=True),
	}

url()

class brand(osv.osv):
	_name = "it.brand"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Brand name', size=32, required=True),
		'url_id': fields.many2many('it.url', 'it_brand_url_rel', 'brand_id', 'url_id', 'Link'),
		'contact_id': fields.many2one('res.partner.address', 'Contact'),
	}

brand()

class users(osv.osv):
	_name = "it.users"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Username', size=32, required=True),
		'contact_id': fields.many2one('res.partner.address', 'User contact'),
	}

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['name','contact_id'], context=context):
			if r['contact_id']:
				name = r['name']+', '+r['contact_id'][1]
			else:
				name = r['name']
			res.append((r['id'], name))
		return res

users()

# Izejmateriali

class cons_domain(osv.osv):
	_name = "it.cons_domain"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Type', size=32, required=True),
	}

cons_domain()

class cons_type(osv.osv):
	_name = "it.cons_type"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Type', size=64, required=True),
		'domain_id': fields.many2one('it.cons_domain', 'Consumable domain', required=True),
	}

cons_type()

class cons_prop(osv.osv):
	_name = "it.cons_prop"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Property name', size=32, required=True),
		'domain_id': fields.many2one('it.cons_domain', 'Consumable domain', required=True),
		'prop_val': fields.one2many('it.cons_prop_val', 'property_id', 'Property value'),
	}

cons_prop()

class cons_prop_val(osv.osv):
	_name = "it.cons_prop_val"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Property value', size=32, required=True),
		'property_id': fields.many2one('it.cons_prop', 'Property'),
	}

cons_prop_val()

class properties(osv.osv):
	_name = "it.properties"
	_rec_name = "name"

	_columns = {
		'name': fields.many2one('it.cons_prop', 'Property name', required=True),
		'values': fields.many2one('it.cons_prop_val','Property value', domain="[('property_id', '=', name)]"),
		#'consum_id': fields.many2one('it.consumables', 'Consumables')
	}

properties()

class consumables(osv.osv):
	_name = "it.consumables"
	_rec_name = "name"

	_columns = {
		'name': fields.char('OEM Model', size=32, required=True),
		'brand_id': fields.many2one('it.brand', 'Brand'),
		'alternatives': fields.many2many('it.consumables', 'it_consumables_rel', 'consum_id', 'alter_id', 'Alternatives'),
		'type': fields.many2many('it.cons_type', 'it_cons_type_rel', 'consum_id', 'type_id', 'Consumable type'),
		'notes': fields.text('Notes'),
		'domain': fields.many2one('it.cons_domain', 'Consumable domain', required=True),
		'property_id': fields.one2many('it.properties','consum_id', 'Properties', ondelete='cascade'),
	}

consumables()
