##############################################################################
#
# Copyright (c) 2008-2009 SIA "KN dati". (http://kndati.lv) All Rights Reserved.
#                    General contacts <info@kndati.lv>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import fields,osv
import tools
import re
from mx.DateTime import RelativeDateTime, now, DateTime, localtime

def _get_net_type(self, cr, uid, context={}):
	obj = self.pool.get('it.net_type')
	ids = obj.search(cr, uid, [])
	res = obj.read(cr, uid, ids, ['type'], context)
	return [(r['type'], r['type']) for r in res]

class network_types(osv.osv):
	_name = "it.net_type"
	_rec_name = "type"

	_columns = {
		'type': fields.char('Type Name', size=10, required=True),
		'description': fields.char('Description', size=64),
	}

network_types()

class network(osv.osv):
	_name = "it.network"
	_rec_name = "name"

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['ip_address', 'name']):
			name = ''
			if r['ip_address']:
				name += r['ip_address'] + ' - '
			name += r['name']
			res.append((r['id'], name))
		return res

	_columns = {
		'name': fields.char('Name', size=100, required=True),
		'owned_by': fields.many2one('res.partner', 'Owner', required=True),
		'resp_admin': fields.many2many('res.partner.address', 'it_network_admin_rel', 'network_id', 'partner_id', 'Responsible admin'),
		'ip_address': fields.char('IP Address', size=15, required=False),
		'netmask' : fields.integer('Netmask', size=3, required=False),
		'notes': fields.text('Notes'),
		'net_type': fields.selection(_get_net_type, 'Network Type'),
		'active': fields.boolean('Active'),
	}

	_defaults = {
		'active' : lambda *a: True,
	}

network()

class ip_reserve(osv.osv):
	_name = "it.reserve"
	_rec_name = "ip_addr_start"

	def _get_gateway(self, cr, uid, ids, field_name, arg=None, context={}):
		res={}
		for p in self.browse(cr, uid, ids, context):
			if p:
				rate=p.netmask
				if rate>=0:
					x = 3
					gateway = ""
					while x >= 0:
						x-=1
						if rate >= 0:
							rate = rate-8
						if rate < 0 and rate != False:
							temp = 255
							temp>>=(abs(rate))
							temp<<=(abs(rate))
							if len(gateway)>0:
								gateway = gateway+'.'+str(temp)
							else:
								gateway = str(temp)
							rate = False
						else:
							if len(gateway)>0:
								gateway = gateway+'.'+'255'
							else:
								gateway = '255'
					res[p.id]=gateway
				else:
					res[p.id]=False
		return res

	_columns = {
		'state': fields.selection([('s','single'), ('r','range'), ('m', 'mask')], 'Type', size=3),
		'ip_addr_start': fields.char('Start address', size=15),
		'ip_addr_end': fields.char('End address', size=15, states={'m':[('readonly',True)], 's':[('readonly',True)]}),
		'netmask' : fields.integer('Netmask', states={'r':[('readonly',True)]}),
		'network_id' : fields.many2one('it.network', 'Network'),
		'gateway': fields.many2one('it.reserve', 'Gateway'),
		'hw_id': fields.many2one('it.hardware', 'Hardware'),
		'dot_not': fields.function(_get_gateway, method=True, string='Netmask (dot notation)', type='char'),
	}

	def _constraint1(self, cr, uid, ids):
		value = self.read(cr, uid, ids, ['netmask'])[0]['netmask']

		if value:
			if value < 0 or value > 32:
				raise osv.except_osv('Incorrect netmask!', 'This netmask is not allowed !')

          	return True

	def _constraint2(self, cr, uid, ids):
		start = self.read(cr, uid, ids, ['ip_addr_start'])[0]['ip_addr_start']

		value = ""
		if start:
			result = re.match('(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$', start)
			if result != None:
				x = 0
				while x < 4:
					byte = int(result.group(x+1))
					if byte > 255 or byte < 0:
						raise osv.except_osv('Incorrect address!', 'Allowed value can be from 0 to 255 !')
					if len(value) > 0:
						value += '.'+str(byte)
					else:
						value += str(byte)
					x += 1
			else:
				raise osv.except_osv('Incorrect address!', 'It is a not allowed address format !')

		state = self.read(cr, uid, ids, ['state'])
		if state[0]['state'] != 'r':
			cr.execute("""
					UPDATE it_reserve
					SET ip_addr_start=%s,ip_addr_end=NULL
					WHERE id in (%s)
				   """,(value, ','.join(map(str,ids))))
		else:
			cr.execute("""
					UPDATE it_reserve
					SET ip_addr_start=%s
					WHERE id in (%s)
				   """,(value, ','.join(map(str,ids))))

          	return True

	def _constraint3(self, cr, uid, ids):
		end = self.read(cr, uid, ids, ['ip_addr_end'])[0]['ip_addr_end']
		if not end:
			return True

		value = ""
		result = re.match('(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$', end)
		if result != None:
			x = 0
			while x < 4:
				byte = int(result.group(x+1))
				if byte > 255:
					raise osv.except_osv('Incorrect address!', 'Allowed value can be from 0 to 255 !')
				if len(value) > 0:
					value += '.'+str(byte)
				else:
					value += str(byte)
				x += 1
		else:
			raise osv.except_osv('Incorrect address!', 'It is a not allowed address format !')

		cr.execute("""
				UPDATE it_reserve
				SET ip_addr_end=%s
				WHERE id in (%s)
			   """,(value, ','.join(map(str,ids))))

          	return True

	_defaults = {
		'state' : lambda *a: 's',
	}
	_constraints = [
        	(_constraint1, 'This netmask is not allowed!', ['netmask']),
		(_constraint2, 'Incorrect address!', ['ip_addr_start']),
		(_constraint3, 'Incorrect address!', ['ip_addr_end'])
    	]

ip_reserve()

class ip_bind(osv.osv):
	_name = "it.bind"
        _rec_name = "ip_address"

	_columns = {
		'ip_address': fields.many2one('it.reserve', 'IP Address', required=True),
		'port_number': fields.integer('Port number', size=5, required=True),
	}

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['ip_address','port_number']):
			name = str(r['ip_address'][1])+':'+str(r['port_number'])
			res.append((r['id'], name))
		return res

	_defaults = {
		'port_number' : lambda *a: 0,
	}

ip_bind()

class licence_type(osv.osv):
	_name = "it.licence_type"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Licence type', size=32, required=True),
	}

licence_type()

class key_type(osv.osv):
	_name = "it.key_type"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Key type', size=32, required=True),
	}

key_type()

class software(osv.osv):
	_name = "it.software"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Name', size=100, required=True),
		'lic_type_id': fields.many2one('it.licence_type', 'Licence type'),
		'software_key': fields.char('Software key', size=64),
		'key_type_id': fields.many2one('it.key_type', 'Key type', required=True),
		'category': fields.many2many('it.software.category', 'it_software_category_rel', 'software_id', 'category_id', 'Categories'),
		'notes': fields.text('Notes'),
		'brand_id': fields.many2one('it.brand', 'Brand'),
		'owner_id': fields.many2one('res.partner', 'Owner'),
		'acq_id': fields.many2one('it.acquisition', 'Acquisition'),
		'inst_cnt_val' : fields.integer('Count of installation'),
		'up_date_val': fields.date('Update date'),
		'lic_date_val': fields.date('Licence date'),
		'soft_inst_id': fields.one2many('it.soft_install', 'software_id', 'Connected hardware'),
	}

	_defaults = {
		'inst_cnt_val' : lambda *a: 1,
	}

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['name','brand_id','lic_type_id']):
			if r['brand_id']:
				name = str(r['brand_id'][1])
				name +=' '
				name += str(r['name'])
				if r['lic_type_id']:
					name +=' '
					name += str(r['lic_type_id'][1])
			else:
				name = name = str(r['name'])
				if r['lic_type_id']:
					name +=' '
					name += str(r['lic_type_id'][1])

			res.append((r['id'], name))
		return res

software()

class proto_type(osv.osv):
	_name = "it.proto_type"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Protocol type', size=16, required=True),
		'main_protocol': fields.many2one('it.proto_type', 'Base protocol'),
                'category': fields.many2many('it.proto_type.category', 'it_proto_type_category_rel', 'proto_type_id', 'category_id', 'Categories'),
	}

proto_type()

class connection(osv.osv):
	_name = "it.connection"
	_rec_name = "proto_type"

	_columns = {
		'proto_type': fields.many2one('it.proto_type', 'Protocol type', required=True),
                'ip_bind': fields.many2one('it.bind', 'IP bind', required=True),
	}

connection()

class dns_direction(osv.osv):
	_name = "it.dns_direction"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Name', size=32, required=True),
                'ip_address': fields.many2many('it.reserve', 'it_dns_dir_rel', 'dns_id', 'addr_id', 'IP bind', 'IP address', required=True),
	}

dns_direction()

class service_it(osv.osv):
	_name = "it.service"
	_rec_name = "name"

	_columns = {
		'name': fields.char('Name', size=32, required=True),
		'software': fields.many2one('it.software', 'Software', size=32, required=True),
		'run_hardware': fields.many2many('it.hardware', 'it_service_hw_rel', 'service_id', 'hw_id', 'Hardware', required=True),
		'dns_directed': fields.many2many('it.dns_direction', 'it_service_dns_rel', 'service_id', 'dns_id', 'DNS'),
		'connections': fields.many2many('it.connection', 'it_service_conn_rel', 'service_id', 'connection_id', 'Connections'),
		'dep_on_service': fields.many2many('it.service', 'it_service_dep_on_rel', 'service_id', 'dep_on_id', 'Dep on service'),
		'support_service': fields.many2many('it.service', 'it_service_support_rel', 'service_id', 'support_id', 'Support'),
		'resp_admin': fields.many2many('res.partner.address', 'it_service_contact_rel', 'service_id', 'partner_id', 'Contacts'),
		'category': fields.many2many('it.service.category', 'it_service_category_rel', 'service_id', 'category_id', 'Categories'),
	}

service_it()

class soft_install(osv.osv):
	_name = "it.soft_install"
	_rec_name = "software_id"

	_columns = {
		'inst_date': fields.date('Installation date', required=True),
		'software_id': fields.many2one('it.software', 'Software', ondelete='cascade'),
		'hardware_id': fields.many2one('it.hardware', 'Hardware', ondelete='cascade'),
	}

	_defaults = {
		'inst_date' : lambda *a: now().strftime('%Y-%m-%d'),
	}

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['software_id','hardware_id']):
			if r['software_id']:
				name = str(r['software_id'])
				if r['hardware_id']:
					name +=', '
					name += str(r['hardware_id'])
			else:
				name = str(r['hardware_id'])
				if r['software_id']:
					name +=', '
					name += str(r['software_id'])
			res.append((r['id'], name))
		return res

soft_install()

class all_hw_by_network(osv.osv):
	_name = "it.all_hw_by_network"
	_auto = False

	_columns = {
		'name': fields.char('Name', size=32, required=True),
		'owned_by': fields.many2one('res.partner', 'Owner', required=True),
		'resp_admin': fields.many2many('res.partner.address', 'it_network_admin_rel', 'network_id', 'partner_id', 'Responsible admin', required=True),
		'ip_address': fields.char('IP Address', size=15, required=False),
		'netmask' : fields.integer('Netmask', size=3, required=False),
	}

	def init(self, cr):
		cr.execute("""
		create or replace view it_all_hw_by_network as (
			SELECT DISTINCT C.*
			FROM it_hardware A, it_reserve B, it_network C
			WHERE A.id = B.hw_id and B.network_id = C.id
		)""")

all_hw_by_network()
